/*
    Capture the Flag - UO Mini-Game
    Copyright (C) 2006 Jason Miller

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

Use uo;
Use os;
Use cfgfile;

Include "include/client";
Include "include/spelldata";
Include "include/res";
Include "include/ranksystem";
Include "include/factions";
Include "include/yesno";
Include "include/dismount";
Include "include/arrays";
Include "util/bank";

Const TEAMID_RED := "CTFRed";
Const TEAMID_BLUE := "CTFBlue";
Const TEAMID_HUERED := 2576;
Const TEAMID_HUEBLUE := 2573;
Const CFTID_INGAME := "IsInCTFMatch";
Const CTFID_INPROGRESS := "CTFMatchInProgress";
Const CTFID_POINTSTOWIN := 3;
Const CTFID_COINREWARDAMT := 2;
Const CTF_CFG_ARENAS := "ctfarenas";
Const CTF_CFG_ARENAS2 := ":unrealctf:ctfarenas";

Function ReadyCFTContestants( AllContestants )

    Set_Critical( 1 );
    Var Cntstnt;

    Foreach Contestant in AllContestants
    Cntstnt := SystemFind( Contestant );
    CheckContestant( Cntstnt );
    DressConstestant( Cntstnt );
    SendContestantToBase( Cntstnt );
    SleepMS( 10 );
    Endforeach
    
    Set_Critical( 0 );

Endfunction

Function RemoveCFTContestants( AllContestants )

    Set_Critical( 1 );

    Var Cntstnt;

    Foreach Contestant in AllContestants
    Cntstnt := SystemFind( Contestant );
    RemoveContestant( Cntstnt );
    SleepMS( 10 );
    Endforeach

    Set_Critical( 0 );

Endfunction

Function CheckContestants( Group )

   If( Group.Size() == 0 || !Group || Group.Size() < 0 )
   Return Error;
   Else
   Return 1;
   Endif

Endfunction

Function EndCTFGame( AllContestants )

    BroadcastToPlayers( "Ending match." );
    Sleep( 5 );
    BroadcastToCTFPlayers( "If you're found with any CTF items you will be banned." );
    Sleep( 5 );
    BroadcastToCTFPlayers( "If you're found with any CTF items you will be banned." );
    Sleep( 5 );
    
    DestroyFlags();
    RemoveCFTContestants( AllContestants );
    EraseGlobalProperty( "CTFStone" );
    EraseGlobalProperty( "CTFRedScore" );
    EraseGlobalProperty( "CTFBlueScore" );
    EraseGlobalProperty( "Red_Contestants" );
    EraseGlobalProperty( "Blue_Contestants" );
    EraseGlobalProperty( "Contestants" );
    EraseGlobalProperty( CTFID_INPROGRESS );
    BroadcastToPlayers( "Match ended." );

Endfunction

Function DressConstestant( Who )

    Var CTFMage := GetObjProperty( Who, "IsMage" );
    Var CTFWarrior := GetObjProperty( Who, "IsWarrior" );
    Var CTFRanger := GetObjProperty( Who, "IsRanger" );
    Var CTFCrafter := GetObjProperty( Who, "IsCrafter" );
    Var CTFBard := GetObjProperty( Who, "IsBard" );
    Var CTFThief := GetObjProperty( Who, "IsThief" );
    Var CTF_Team := GetObjProperty( Who, CFTID_INGAME );
    Var CTF_Color := 1157;

    If( Who.dead )
    ResurrectPlayer( Who );
    Endif
    
    Case( CTF_Team )
    TEAMID_RED:  CTF_Color := TEAMID_HUERED;
    TEAMID_BLUE: CTF_Color := TEAMID_HUEBLUE;
    Endcase
    
    If( CTFMage )
    DressMage( who, CTF_Color );
    Elseif( CTFWarrior )
    DressWarrior( who, CTF_Color );
    Elseif( CTFRanger )
    DressRanger( who, CTF_Color );
    Elseif( CTFBard )
    DressBard( who, CTF_Color );
    Else
    DressPP( who, CTF_Color );
    Endif
    
    SetObjProperty( Who, CASTINGOPT_PROP_NAME, CASTINGOPT_NO_REAGENTS );

Endfunction

Function BroadcastToCTFPlayers( Message );

    Var CTF_Contestants := GetGlobalProperty( "Contestants" );
    Var Cntstnt;

    Foreach Contestant in CTF_Contestants
    Cntstnt := SystemFind( Contestant );
    SendSysMessage( Cntstnt, "[CTF System] " + CStr( Message ), FONT_NORMAL, 2479 );
    SleepMS( 10 );
    Endforeach
    
    Return 1;

Endfunction

Function BroadcastToPlayers( Message );

    Foreach Player in EnumerateOnlineCharacters()
    If( GetObjProperty( Player, "CTF_Chat" ) == "ON" || !GetObjProperty( Player, "CTF_Chat" ) )
    SendSysMessage( Player, "[CTF System] " + CStr( Message ), FONT_NORMAL, 2479 );
    Endif
    Endforeach

    Return 1;

Endfunction

Function SystemFind( xSerial )

    Var What := SystemFindObjectBySerial( xSerial[1] );
    
    If( What == error )
    Return Error;
    Endif

    Return What;

Endfunction

Function StartCTFMatch()

    Var CTFCFG_Arenas := ReadConfigFile( CTF_CFG_ARENAS );
    Var Arena := 1;
    Var Red_Flag_StartLoc := CTFCFG_Arenas[Arena].RedFlag;
    Var Blue_Flag_StartLoc := CTFCFG_Arenas[Arena].BlueFlag;
    Red_Flag_StartLoc := SplitWords( Red_Flag_StartLoc );
    Blue_Flag_StartLoc := SplitWords( Blue_Flag_StartLoc );
    Var Flag_Type := 0x7032;
    Var Red_Flag;
    Var Blue_Flag;
    
    DestroyFlags();
    
    BroadcastToCTFPlayers( "60 seconds until the flags are placed at their bases." );
    Sleep( 30 );
    BroadcastToCTFPlayers( "30 seconds left until the flags are placed at their bases." );
    Sleep( 15 );
    BroadcastToCTFPlayers( "15 seconds left until the flags are placed at their bases." );
    Sleep( 15 );
    
    Red_Flag := CreateItemAtLocation( CInt( Red_Flag_StartLoc[1] ), CInt( Red_Flag_StartLoc[2] ), CInt( Red_Flag_StartLoc[3] ), Flag_Type, 1 );
    SetObjProperty( Red_Flag, "IsAtHomeBase", 1 );
    SetObjProperty( Red_Flag, "FlagHome", { CInt( Red_Flag_StartLoc[1] ), CInt( Red_Flag_StartLoc[2] ), CInt( Red_Flag_StartLoc[3] ) } );
    SetObjProperty( Red_Flag, CFTID_INGAME, TEAMID_RED );
    Red_Flag.color := TEAMID_HUERED;
    Red_Flag.name := "Red Teams Flag";
    
    Blue_Flag := CreateItemAtLocation( CInt( Blue_Flag_StartLoc[1] ), CInt( Blue_Flag_StartLoc[2] ), CInt( Blue_Flag_StartLoc[3] ), Flag_Type, 1 );
    SetObjProperty( Blue_Flag, "IsAtHomeBase", 1 );
    SetObjProperty( Blue_Flag, "FlagHome", { CInt( Blue_Flag_StartLoc[1] ), CInt( Blue_Flag_StartLoc[2] ), CInt( Blue_Flag_StartLoc[3] ) } );
    SetObjProperty( Blue_Flag, CFTID_INGAME, TEAMID_BLUE );
    Blue_Flag.color := TEAMID_HUEBLUE;
    Blue_Flag.name := "Blue Teams Flag";
    
    SetGlobalProperty( "CTFFlags", { Red_Flag.serial, Blue_Flag.serial } );
    
    BroadcastToCTFPlayers( "The flags have been placed at their bases!" );
    
Endfunction

Function SelectTeams()

    Var CTF_Red_Contestants := Array;
    Var CTF_Blue_Contestants := Array;
    Var CTF_Contestants := Array;
    Var Chaos_Queued := GetGlobalProperty( "QueuedChaos_Players" );
    Var Order_Queued := GetGlobalProperty( "QueuedOrder_Players" );
    Var QChaos_Count := Chaos_Queued.Size();
    Var QOrder_Count := Order_Queued.Size();
    Var TSize := ( QChaos_Count + QOrder_Count / 2 );

    Var iR := 0;
    Repeat
    iR := iR + 1;
    Var Red_Player := SystemFind( Chaos_Queued[iR] );
    If( Red_Player )
      If( (Red_Player.acct).GetProp( "Release" ) == 0 )
      CTF_Red_Contestants.Append( Chaos_Queued[iR] );
      CTF_Contestants.Append( Chaos_Queued[iR] );
      SetObjProperty( Red_Player, CFTID_INGAME, TEAMID_RED );
      Endif
    Endif
    SleepMS( 10 );
    Until( iR >= TSize );
    
    Var iB := 0;
    Repeat
    iB := iB + 1;
    Var Blue_Player := SystemFind( Order_Queued[iB] );
    If( Blue_Player )
      If( (Blue_Player.acct).GetProp( "Release" ) == 0 )
      CTF_Blue_Contestants.Append( Order_Queued[iB] );
      CTF_Contestants.Append( Order_Queued[iB] );
      SetObjProperty( Blue_Player, CFTID_INGAME, TEAMID_BLUE );
      Endif
    Endif
    SleepMS( 10 );
    Until( iB >= TSize );
    
    SetGlobalProperty( "CTFRedScore", 0 );
    SetGlobalProperty( "CTFBlueScore", 0 );
    SetGlobalProperty( "Red_Contestants", CTF_Red_Contestants );
    SetGlobalProperty( "Blue_Contestants", CTF_Blue_Contestants );
    SetGlobalProperty( "Contestants", CTF_Contestants );
    SetGlobalProperty( CTFID_INPROGRESS, 1 );
    
    ReadyCFTContestants( CTF_Contestants );
    
    Return 1;

Endfunction

Function UnequipFlag( Who, Flag );

    Set_Critical( 1 );

    SetObjProperty( Flag, "DroppedCTFFlag", 1 );
    EraseObjProperty( Who, "HasCTFFlag" );

    If( Flag.movable )
    MoveItemToLocation( Flag, Who.x, Who.y, Who.z, MOVEITEM_FORCELOCATION );
    Flag.graphic := 0x1640;
    Flag.movable := 0;
    Set_Critical( 0 );
    Return 1;
    Elseif( !Flag.movable )
    Flag.movable := 1;
    MoveItemToLocation( Flag, Who.x, Who.y, Who.z, MOVEITEM_FORCELOCATION );
    Flag.graphic := 0x1640;
    Flag.movable := 0;
    Set_Critical( 0 );
    Return 1;
    Endif
    
    Set_Critical( 0 );

    Return Error;
    
Endfunction

Function EquipFlag( Who, Flag );

    Set_Critical( 1 );

    SetObjProperty( Who, "HasCTFFlag", Flag.serial );
    EraseObjProperty( Flag, "IsAtHomeBase" );
    EraseObjProperty( Flag, "DroppedCTFFlag" );

    If( Flag.movable )
    Flag.graphic := 0x1515;
    Flag.newbie := 1;
    EquipItem( Who, Flag );
    Flag.movable := 0;
    Set_Critical( 0 );
    Return 1;
    Elseif( !Flag.movable )
    Flag.movable := 1;
    Flag.graphic := 0x1515;
    Flag.newbie := 1;
    EquipItem( Who, Flag );
    Flag.movable := 0;
    Set_Critical( 0 );
    Return 1;
    Endif
    
    Set_Critical( 0 );

    Return Error;

Endfunction

Function ResurrectPlayer( Who )

    If( Who.dead )
    Resurrect( who );
    FixRaceColor( Who );
    SetStamina( Who, Cint( 35 + Random( 25 ) ) );
    SetHp( Who, Cint( 35 + Random( 25 ) ) );
    SetMana( Who, Cint( 35 + Random( 25 ) ) );
      Var DeathRobe := GetEquipmentByLayer( Who, LAYER_ROBE );
      If( DeathRobe )
      DestroyItem( DeathRobe );
      Endif
    Foreach xItem in EnumerateItemsInContainer( Who.backpack )
    If( GetObjProperty( xItem, "CTFRobe" ) )
    EquipItem( Who, xItem );
    Break;
    Endif
    Endforeach
    Else
    Return Error;
    Endif

Endfunction

Function DestroyFlags()

    Var Flags := GetGlobalProperty( "CTFFlags" );
    If( Flags )
    Var Red_Flag := SystemFind( Flags[1] );
    Var Blue_Flag := SystemFind( Flags[2] );
    DestroyItem( Red_Flag );
    DestroyItem( Blue_Flag );
    Endif

Endfunction

Function SendContestantToBase( Who )

    Var CTFCFG_Arenas := ReadConfigFile( CTF_CFG_ARENAS2 );
    Var iArena := 1;
    Var CTF_Team := GetObjProperty( Who, CFTID_INGAME );
    If( CTF_Team == error )
    Print( "SCTB>CTF_TEAM: " + CTF_Team.errortext );
    Return Error;
    Endif

    If( CTF_Team == TEAMID_RED )
    
    Var Red_Player_StartLoc := CTFCFG_Arenas[iArena].Red_Start_Loc;
    
    If( Red_Player_StartLoc == error )
    Print( "Red_Player_StartLoc: " + Red_Player_StartLoc.errortext );
    Return Error;
    Endif
    
    Red_Player_StartLoc := SplitWords( Red_Player_StartLoc );
    MoveCharacterToLocation( Who, CInt( Red_Player_StartLoc[1] ), CInt( Red_Player_StartLoc[2] ), CInt( Red_Player_StartLoc[3] ), MOVECHAR_FORCELOCATION );
    Return 1;

    Elseif( CTF_Team == TEAMID_BLUE )

    Var Blue_Player_StartLoc := CTFCFG_Arenas[iArena].Blue_Start_Loc;
    
    If( Blue_Player_StartLoc == error )
    Print( "Blue_Player_StartLoc: " + Blue_Player_StartLoc.errortext );
    Return Error;
    Endif
    
    Blue_Player_StartLoc := SplitWords( Blue_Player_StartLoc );
    MoveCharacterToLocation( Who, CInt( Blue_Player_StartLoc[1] ), CInt( Blue_Player_StartLoc[2] ), CInt( Blue_Player_StartLoc[3] ), MOVECHAR_FORCELOCATION );
    Return 1;
    Endif

Endfunction

Function RemoveContestant( Who )

    Var CTFCheck := GetObjProperty( Who, CFTID_INGAME );
    Var CTFCFG_Arenas := ReadConfigFile( CTF_CFG_ARENAS );
    Var CTF_Red_Contestants := GetGlobalProperty( "Red_Contestants" );
    Var CTF_Blue_Contestants := GetGlobalProperty( "Blue_Contestants" );
    Var CTF_Contestants := GetGlobalProperty( "Contestants" );

    If( CTFCheck )

    Var Hair := GetEquipmentByLayer( Who, LAYER_HAIR );
    Var Beard := GetEquipmentByLayer( Who, LAYER_BEARD );

    Foreach Worn in ListEquippedItems( Who )
    If( Worn && Worn != Hair && Worn != Beard && Worn != Who.backpack )
      If( GetObjProperty( Worn, "CTF" ) || GetObjProperty( Worn, "CTFRobe" ) )
      DestroyEquipment( Who, Worn );
      Endif
    Endif
    Endforeach
    
    Foreach Item in EnumerateItemsInContainer( Who.backpack );
    If( GetObjProperty( Item, "CTF" ) || GetObjProperty( Item, "CTFRobe" ) )
    DestroyEquipment( Who, Item );
    Endif
    Endforeach
    
    RemoveFromArray( CTF_Red_Contestants, Who.serial );
    RemoveFromArray( CTF_Blue_Contestants, Who.serial );
    RemoveFromArray( CTF_Contestants, Who.serial );
    
    SetGlobalProperty( "Red_Contestants", CTF_Red_Contestants );
    SetGlobalProperty( "Blue_Contestants", CTF_Blue_Contestants );
    SetGlobalProperty( "Contestants", CTF_Contestants );
    
    SendPlayerToRaceHomeTown( Who );

    EraseObjProperty( Who, CASTINGOPT_PROP_NAME );
    EraseObjProperty( Who, CFTID_INGAME );
    EraseObjProperty( Who, "HasCTFFlag" );

    Endif

Endfunction

Function CanAdd( who, team1, team2, add_mob )

    If( !add_mob )
    Return Error;
    Elseif( add_mob && !add_mob.ip )
    SendSysMessage( who, "That is not an online player." );
    Return Error;
    Elseif( add_mob.serial in team1 )
    SendSysMessage( who, add_mob.name + " is already on the Red Team." );
    Return Error;
    Elseif( add_mob.serial in team2 )
    SendSysMessage( who, add_mob.name + " is already on the Blue Team." );
    Return Error;
    Else
    Return 1;
    Endif

Endfunction

Function CheckContestant( Who )

    Var Hair := GetEquipmentByLayer( Who, LAYER_HAIR );
    Var Beard := GetEquipmentByLayer( Who, LAYER_BEARD );
    Var Item_Count := 0;
    Var PassGo := { LAYER_HAIR, LAYER_BEARD };

    Var Mount := GetEquipmentByLayer( Who, LAYER_MOUNT );
    
    If( Mount )
    Dismount( Who );
    Endif

    Foreach Worn in ListEquippedItems( Who )
    If( Worn && Worn != Hair && Worn != Beard && Worn != Who.backpack )
    Var xBankbox := FindBankBox( Who );
    MoveItemToContainer( Worn, xBankbox );
    SleepMS( 5 );
    Endif
    Endforeach

    Foreach Item in EnumerateItemsInContainer( Who.backpack );
    Var xBankbox := FindBankBox( Who );
    MoveItemToContainer( Item, xBankbox );
    SleepMS( 10 );
    Endforeach
    
    SendSysMessage( Who, "Everything you had on you was moved into your backbox.", FONT_NORMAL, 2611 );
    
    Return 1;

Endfunction

Function RewardCTFTeam( CTFTeam )
    
    Set_Critical( 1 );
    Var Cntstnt;

    Foreach Contestant in CTFTeam
    Cntstnt := SystemFind( Contestant );
    Var xBankbox := FindBankBox( Cntstnt );
    CreateItemInContainer( xBankbox, 0xA396, CTFID_COINREWARDAMT );
    Var CTF_Wins := GetObjProperty( Cntstnt, "LT_CTFWins" );
    If( !CTF_Wins )
    CTF_Wins := 0;
    Endif
    SetObjProperty( Cntstnt, "LT_CTFWins", CTF_Wins + 1 );
    SendSysMessage( Cntstnt, "Congrats, You've earned some unreal coins!", FONT_NORMAL, 2600 );
    SleepMS( 10 );
    Endforeach

    Set_Critical( 0 );

Endfunction

Function AwardCTFPoint( Who, Points )

    Var Captures := GetObjProperty( Who, "LT_Captures" );
    If( !Captures )
    Captures := 0;
    Endif
    
    SetObjProperty( Who, "LT_Captures", Captures + Points );
    
    Return 1;

Endfunction

Function LookForFlag( Who, Flag )

    Var Flag_Team := GetObjProperty( Flag, CFTID_INGAME );
    
    Foreach Item in ListItemsNearLocation( Who.x, Who.y, Who.z, 2 );
    Var OFlag_Team := GetObjProperty( Item, CFTID_INGAME );
    Var AtBase := GetObjProperty( Item, "IsAtHomeBase" );
    If( OFlag_Team )
      If( Flag_Team == TEAMID_RED && OFlag_Team == TEAMID_BLUE && AtBase )
      Return 1;
      Endif
      If( Flag_Team == TEAMID_BLUE && OFlag_Team == TEAMID_RED && AtBase )
      Return 1;
      Endif
    Endif
    Endforeach
    
    Return Error;

Endfunction

Function DestroyEquipment( Who, Item )

    Var Cursed := GetObjProperty( Item, "Cursed" );

    If( Cursed )
    SetObjProperty( Item, "Cursed", 3 );
    Endif

    If( Who.backpack )
    MoveItemToContainer( Item, Who.backpack );
    Endif

    DestroyItem( item );

Endfunction
/*
Function RemoveFromArray( byref x_array, match )

    Foreach index in ( x_array )
    If ( index == match )
    x_array.Erase( _index_iter );
    Return 1;
    Endif
    SleepMS( 2 );
    Endforeach
    Return Error;

Endfunction
*/
Function DressMage( player, TeamColor )
	var ccode := TeamColor;
    	var hat := CreateItemInBackpack( player, 0x1718, 1);
	hat.color := ccode;
        hat.newbie := 1;
	SetObjProperty(hat,"CTF",1);
    	var robe := CreateItemInBackpack( player, 0x1F03, 1);
	robe.color := ccode;
	EquipItem( player, robe );
        robe.newbie := 1;
	SetObjProperty(robe,"CTFRobe",1);
    	var bandies := CreateItemInBackpack( player, 0xe21, 500);
	bandies.color := ccode;
        bandies.newbie := 1;
	SetObjProperty(bandies,"CTF",1);
	var book := CreateItemInBackpack(player,0x0efa, 1);
	book.color := ccode;
	SetObjProperty(book,"CTF",1);
	var i;
	for (i := 0x1f2d; i <= 0x1f64; i := i + 1)
		if (spellok ( i )  )
			CreateItemInContainer(book,  i, 1);
		endif
	endfor
    	var staff := CreateItemInBackpack( player, 0xe8a, 1);
	staff.color := ccode;
        staff.newbie := 1;
	SetObjProperty(staff,"CTF",1);
	staff.dmg_mod := 20;
	staff.name := "a Quarterstaff of Power[CTF]";
	var bow := CreateItemInBackpack( player, 0x7600, 1);
	bow.color := ccode;
        bow.newbie := 1;
	SetObjProperty(bow,"CTF",1);
	bow.dmg_mod := 15;
	bow.name := "Bow of Power[CTF]";
	var arrow := CreateItemInBackpack( player, 0xf3f, 1000);
	arrow.color := ccode;
        arrow.newbie := 1;
	SetObjProperty(arrow,"CTF",1);
	var boir := CreateItemInBackpack( player, 0xba30, 1 );
	boir.color := ccode;
        boir.newbie := 1;
	SetObjProperty( boir,"CTF",1);
Endfunction

Function DressBard( player, TeamColor )

    Var ccode := TeamColor;
    Var hat := CreateItemInBackpack( player, 0x1718, 1 );
    hat.color := ccode;
    hat.newbie := 1;
    SetObjProperty( hat, "CTF", 1 );
    Var robe := CreateItemInBackpack( player, 0x1F03, 1 );
    robe.color := ccode;
    EquipItem( player, robe );
    robe.newbie := 1;
    SetObjProperty( robe, "CTFRobe", 1 );
    Var bandies := CreateItemInBackpack( player, 0xe21, 100 );
    bandies.color := ccode;
    bandies.newbie := 1;
    SetObjProperty( bandies, "CTF", 1 );
    // Book of Songs
    Var bard := CreateItemInBackpack( player, 0x6177, 1 );
    SetObjProperty( bard , "Lesser" , 255 );
    SetObjProperty( bard , "Greater" , 255 );
    bard.color := ccode;
    SetObjProperty( bard, "CTF", 1 );
    Var staff := CreateItemInBackpack( player, 0xe8a, 1 );
    staff.color := ccode;
    staff.newbie := 1;
    SetObjProperty( staff, "CTF", 1 );
    staff.dmg_mod := 20;
    staff.name := "a Quarterstaff of Power[CTF]";
    Var bow := CreateItemInBackpack( player, 0x7600, 1 );
    bow.color := ccode;
    bow.newbie := 1;
    SetObjProperty( bow, "CTF", 1 );
    bow.dmg_mod := 15;
    bow.name := "Bow of Power[CTF]";
    Var arrow := CreateItemInBackpack( player, 0xf3f, 1000 );
    arrow.color := ccode;
    arrow.newbie := 1;
    SetObjProperty( arrow, "CTF", 1 );
    Var boir := CreateItemInBackpack( player, 0xba30, 1 );
    boir.color := ccode;
    boir.newbie := 1;
    SetObjProperty( boir, "CTF",1 );
    Var harp := CreateItemInBackpack( player, 0xEB2, 1 );
    harp.color := ccode;
    harp.newbie := 1;
    SetObjProperty( harp, "CTF",1 );
    
Endfunction

Function DressWarrior( player, TeamColor )
	var ccode := TeamColor;
    	var robe := CreateItemInBackpack( player, 0x1F03, 1);
	robe.color := ccode;
	EquipItem( player, robe );
        robe.newbie := 1;
	SetObjProperty( robe, "CTFRobe", 1 );
    	var arms := CreateItemInBackpack( player, 0x1410, 1);
	arms.color := ccode;
        arms.newbie := 1;
	SetObjProperty(arms,"CTF",1);
    	var legs := CreateItemInBackpack( player, 0x1411, 1);
	legs.color := ccode;
        legs.newbie := 1;
	SetObjProperty(legs,"CTF",1);
    	var helm := CreateItemInBackpack( player, 0x1412, 1);
	helm.color := ccode;
        helm.newbie := 1;
	SetObjProperty(helm,"CTF",1);
    	var gorg := CreateItemInBackpack( player, 0x1413, 1);
	gorg.color := ccode;
        gorg.newbie := 1;
	SetObjProperty(gorg,"CTF",1);
    	var glove := CreateItemInBackpack( player, 0x1414, 1);
	glove.color := ccode;
        glove.newbie := 1;
	SetObjProperty(glove,"CTF",1);
	var bp := CreateItemInBackpack( player, 0x1415, 1);
	bp.color := ccode;
        bp.newbie := 1;
	SetObjProperty(bp,"CTF",1);
    	var heater := CreateItemInBackpack( player, 0x1b76, 1);
	heater.color := ccode;
        heater.newbie := 1;
	SetObjProperty(heater,"CTF",1);
    	var bandies := CreateItemInBackpack( player, 0xe21, 500 );
	bandies.color := ccode;
        bandies.newbie := 1;
	SetObjProperty(bandies,"CTF",1);
	var kryss := CreateItemInBackpack( player, 0x1401, 1);
	kryss.color := ccode;
	kryss.dmg_mod := 20;
        kryss.newbie := 1;
	SetObjProperty(kryss,"CTF",1);
	kryss.name := "Kryss of Power[CTF]";
    	var mace := CreateItemInBackpack( player, 0xf5c, 1);
	mace.color := ccode;
	SetObjProperty(mace,"CTF",1);
	mace.dmg_mod := 20;
        mace.newbie := 1;
	mace.name := "Mace of Power[CTF]";
    	var sword := CreateItemInBackpack( player, 0xf5e, 1);
	sword.color := ccode;
        sword.newbie := 1;
	SetObjProperty(sword,"CTF",1);
	sword.dmg_mod := 20;
	sword.name := "Broad Sword of Power[CTF]";
    	var spear := CreateItemInBackpack( player, 0xf63, 1);
	spear.color := ccode;
        spear.newbie := 1;
	SetObjProperty(spear,"CTF",1);
	spear.dmg_mod := 20;
	spear.name := "Spear of Power[CTF]";
	var book := CreateItemInBackpack(player,0x0efa, 1);
	book.color := ccode;
	SetObjProperty(book,"CTF",1);
	var i;
	for (i := 0x1f2d; i <= 0x1f64; i := i + 1)
		if (spellok ( i )  )
			CreateItemInContainer(book,  i, 1);
		endif
	endfor
	var bow := CreateItemInBackpack( player, 0x7600, 1);
	bow.color := ccode;
        bow.newbie := 1;
	SetObjProperty(bow,"CTF",1);
	bow.dmg_mod := 15;
	bow.name := "Bow of Power[CTF]";
	var arrow := CreateItemInBackpack( player, 0xf3f, 1000);
	arrow.color := ccode;
        arrow.newbie := 1;
	SetObjProperty(arrow,"CTF",1);
	var boir := CreateItemInBackpack( player, 0xba30, 1 );
	boir.color := ccode;
        boir.newbie := 1;
	SetObjProperty( boir,"CTF",1);
Endfunction

Function DressRanger( player, TeamColor )
	var ccode := TeamColor;
    	var robe := CreateItemInBackpack( player, 0x1F03, 1);
	robe.color := ccode;
	EquipItem( player, robe );
        robe.newbie := 1;
	SetObjProperty( robe, "CTFRobe", 1 );
    	var arms := CreateItemInBackpack( player, 0x13ee, 1);
	arms.color := ccode;
        arms.newbie := 1;
	SetObjProperty(arms,"CTF",1);
    	var legs := CreateItemInBackpack( player, 0x13be, 1);
	legs.color := ccode;
        legs.newbie := 1;
	SetObjProperty(legs,"CTF",1);
    	var helm := CreateItemInBackpack( player, 0x13c0, 1);
	helm.color := ccode;
        helm.newbie := 1;
	SetObjProperty(helm,"CTF",1);
    	var glove := CreateItemInBackpack( player, 0x13f2, 1);
	glove.color := ccode;
        glove.newbie := 1;
	SetObjProperty(glove,"CTF",1);
	var bp := CreateItemInBackpack( player, 0x13bf, 1);
	bp.color := ccode;
        bp.newbie := 1;
	SetObjProperty(bp,"CTF",1);
	var boots := CreateItemInBackpack( player, 0x170b, 1);
	boots.color := ccode;
        boots.newbie := 1;
	SetObjProperty(boots,"CTF",1);
    	var bandies := CreateItemInBackpack( player, 0xe21, 100);
	bandies.color := ccode;
        bandies.newbie := 1;
	SetObjProperty(bandies,"CTF",1);
	var bow := CreateItemInBackpack( player, 0x7600, 1);
	bow.color := ccode;
        bow.newbie := 1;
	SetObjProperty(bow,"CTF",1);
	bow.dmg_mod := 25;
	bow.name := "Bow of Vanq[CTF]";
	var arrow := CreateItemInBackpack( player, 0xf3f, 2000 );
	arrow.color := ccode;
        arrow.newbie := 1;
	SetObjProperty(arrow,"CTF",1);
	var pot;
	var potloop := 0;
	while (potloop < 5)
		potloop := potloop + 1;
		pot := CreateItemInBackpack (player ,0xdc02, 1);
          pot.newbie := 1;
		SetObjProperty(pot,"CTF",1);
	endwhile
 
	var book := CreateItemInBackpack(player,0x0efa, 1);
	book.color := ccode;
	SetObjProperty(book,"CTF",1);
	var i;
	for (i := 0x1f2d; i <= 0x1f64; i := i + 1)
		if (spellok ( i )  )
			CreateItemInContainer(book,  i, 1);
		endif
	endfor
	var boir := CreateItemInBackpack( player, 0xba30, 1 );
	boir.color := ccode;
        boir.newbie := 1;
	SetObjProperty( boir,"CTF",1);
Endfunction

Function DressPP( player, TeamColor )
	var ccode := TeamColor;
    	var robe := CreateItemInBackpack( player, 0x1F03, 1);
	robe.color := ccode;
	EquipItem( player, robe );
        robe.newbie := 1;
	SetObjProperty( robe, "CTFRobe", 1 );
    	var arms := CreateItemInBackpack( player, 0x1410, 1);
	arms.color := ccode;
        arms.newbie := 1;
	SetObjProperty(arms,"CTF",1);
    	var legs := CreateItemInBackpack( player, 0x1411, 1);
	legs.color := ccode;
        legs.newbie := 1;
	SetObjProperty(legs,"CTF",1);
    	var helm := CreateItemInBackpack( player, 0x1412, 1);
	helm.color := ccode;
        helm.newbie := 1;
	SetObjProperty(helm,"CTF",1);
    	var gorg := CreateItemInBackpack( player, 0x1413, 1);
	gorg.color := ccode;
        gorg.newbie := 1;
	SetObjProperty(gorg,"CTF",1);
    	var glove := CreateItemInBackpack( player, 0x1414, 1);
	glove.color := ccode;
        glove.newbie := 1;
	SetObjProperty(glove,"CTF",1);
	var bp := CreateItemInBackpack( player, 0x1415, 1);
	bp.color := ccode;
        bp.newbie := 1;
	SetObjProperty(bp,"CTF",1);
    	var heater := CreateItemInBackpack( player, 0x1b76, 1);
	heater.color := ccode;
        heater.newbie := 1;
	SetObjProperty(heater,"CTF",1);
    	var bandies := CreateItemInBackpack( player, 0xe21, 500);
	SetObjProperty(bandies,"CTF",1);
        bandies.newbie := 1;
	bandies.color := ccode;
	var book := CreateItemInBackpack(player,0x0efa, 1);
	book.color := ccode;
	SetObjProperty(book,"CTF",1);
	var i;
	for (i := 0x1f2d; i <= 0x1f64; i := i + 1)
		if (spellok ( i )  )
			CreateItemInContainer(book,  i, 1);
		endif
	endfor
    	var kryss := CreateItemInBackpack( player, 0x1401, 1);
	kryss.color := ccode;
	kryss.dmg_mod := 20;
        kryss.newbie := 1;
	kryss.name := "Kryss of Power[CTF]";
	SetObjProperty(kryss,"CTF",1);
    	var mace := CreateItemInBackpack( player, 0xf5c, 1);
	mace.color := ccode;
	mace.dmg_mod := 20;
        mace.newbie := 1;
	mace.name := "Mace of Power[CTF]";
	SetObjProperty(mace,"CTF",1);
    	var sword := CreateItemInBackpack( player, 0xf5e, 1);
	sword.color := ccode;
	sword.dmg_mod := 20;
        sword.newbie := 1;
	sword.name := "Broad Sword of Power[CTF]";
	SetObjProperty(sword,"CTF",1);
    	var spear := CreateItemInBackpack( player, 0xf63, 1);
	spear.color := ccode;
        spear.newbie := 1;
	SetObjProperty(spear,"CTF",1);
	spear.dmg_mod := 20;
	spear.name := "Spear of Power[CTF]";
	var bow := CreateItemInBackpack( player, 0x7600, 1);
	bow.color := ccode;
        bow.newbie := 1;
	SetObjProperty(bow,"CTF",1);
	bow.dmg_mod := 15;
	bow.name := "Bow of Power[CTF]";
	var arrow := CreateItemInBackpack( player, 0xf3f, 1000 );
	arrow.color := ccode;
        arrow.newbie := 1;
	SetObjProperty(arrow,"CTF",1);
 
	var boir := CreateItemInBackpack( player, 0xba30, 1 );
	boir.color := ccode;
        boir.newbie := 1;
	SetObjProperty( boir,"CTF",1);
 
Endfunction

Function SpellOk( what )

    Case( what )
    0x1F41:
    0x1F42:
    0x1F44:
    0x1F4C:
    0x1F4D:
    0x1F4F:
    0x1F54:
    0x1F60:
    0x1F66:
    0x1F67:
    0x1F68:
    0x1F69:
    0x1F6A:
    0x1F6B:
    0x1F6C: Return 0;
    default: Return 1;
    Endcase

Endfunction
