/*
    Color Wars - UO Mini-Game
    Copyright (C) 2006 Jason Miller

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

Use uo;
Use os;
Use cfgfile;

Include "include/client";
Include "include/spelldata";
Include "include/res";
Include "include/ranksystem";
Include "include/yesno";
Include "include/dismount";
Include "include/arrays";
Include "util/bank";

Const CWID_TEAMID_RED := "CWRed";
Const CWID_TEAMID_BLUE := "CWBlue";
Const CWID_TEAMID_HUERED := 2576;
Const CWID_TEAMID_HUEBLUE := 2573;
Const CWID_INGAME := "IsInCWMatch";
Const CWID_INPROGRESS := "CWMatchInProgress";
Const CWID_COINREWARDAMT := 2;
Const CW_CFG_ARENAS := "cwarenas";
Const CW_CFG_ARENAS2 := ":unrealcw:cwarenas";

Function ReadyCWContestants( AllContestants )

    Set_Critical( 1 );
    Var Cntstnt;

    Foreach Contestant in AllContestants
    Cntstnt := SystemFindCW( Contestant );
    CheckCWContestant( Cntstnt );
    DressCWConstestant( Cntstnt );
    SendContestantToCWBase( Cntstnt );
    SleepMS( 10 );
    Endforeach
    
    Set_Critical( 0 );

Endfunction

Function RemoveCWContestants( AllContestants )

    Set_Critical( 1 );

    Var Cntstnt;

    Foreach Contestant in AllContestants
    Cntstnt := SystemFindCW( Contestant );
    RemoveCWContestant( Cntstnt );
    SleepMS( 10 );
    Endforeach

    Set_Critical( 0 );

Endfunction

Function CheckCWContestants( Group )

   If( Group.Size() == 0 || !Group || Group.Size() < 0 )
   Return Error;
   Else
   Return 1;
   Endif

Endfunction

Function EndCWGame( AllContestants )

    BroadcastCWToPlayers( "Ending match." );
    Sleep( 5 );
    BroadcastToCWPlayers( "If you're found with any CW items you will be banned." );
    Sleep( 5 );
    BroadcastToCWPlayers( "If you have any CW items please throw them away now." );
    Sleep( 5 );

    RemoveCWContestants( AllContestants );
    EraseGlobalProperty( "CW_Red_Contestants" );
    EraseGlobalProperty( "CW_Blue_Contestants" );
    EraseGlobalProperty( "CW_Contestants" );
    EraseGlobalProperty( CWID_INPROGRESS );
    BroadcastCWToPlayers( "Match ended." );

Endfunction

Function DressCWConstestant( Who )

    Var CWMage := GetObjProperty( Who, "IsMage" );
    Var CWWarrior := GetObjProperty( Who, "IsWarrior" );
    Var CWRanger := GetObjProperty( Who, "IsRanger" );
    Var CWCrafter := GetObjProperty( Who, "IsCrafter" );
    Var CWBard := GetObjProperty( Who, "IsBard" );
    Var CWThief := GetObjProperty( Who, "IsThief" );
    Var CW_Team := GetObjProperty( Who, CWID_INGAME );
    Var CW_Color := 1157;
    
    Case( CW_Team )
    CWID_TEAMID_RED:  CW_Color := CWID_TEAMID_HUERED;
    CWID_TEAMID_BLUE: CW_Color := CWID_TEAMID_HUEBLUE;
    Endcase
    
    If( CWMage )
    DressCWMage( who, CW_Color );
    Elseif( CWWarrior )
    DressCWWarrior( who, CW_Color );
    Elseif( CWRanger )
    DressCWRanger( who, CW_Color );
    Elseif( CWBard )
    DressCWBard( who, CW_Color );
    Else
    DressCWPP( who, CW_Color );
    Endif

Endfunction

Function BroadcastToCWPlayers( Message );

    Var CW_Contestants := GetGlobalProperty( "CW_Contestants" );
    Var Cntstnt;

    Foreach Contestant in CW_Contestants
    Cntstnt := SystemFindCW( Contestant );
    SendSysMessage( Cntstnt, "[CW System] " + CStr( Message ), FONT_NORMAL, 2479 );
    SleepMS( 10 );
    Endforeach
    
    Return 1;

Endfunction

Function BroadcastCWToPlayers( Message );

    Foreach Player in EnumerateOnlineCharacters()
    If( GetObjProperty( Player, "CW_Chat" ) == "ON" || !GetObjProperty( Player, "CW_Chat" ) )
    SendSysMessage( Player, "[CW System] " + CStr( Message ), FONT_NORMAL, 2479 );
    Endif
    Endforeach

    Return 1;

Endfunction

Function SystemFindCW( xSerial )

    Var What := SystemFindObjectBySerial( xSerial[1] );
    
    If( What == error )
    Return Error;
    Endif

    Return What;

Endfunction

Function SelectCWTeams( Queued_Players )

    Var CW_Red_Contestants := Array;
    Var CW_Blue_Contestants := Array;
    Var CW_Contestants := Array;
    Var QTotal := Queued_Players.Size();
    Var TSize := ( QTotal / 2 );
    
    Var L_CW := GetGlobalProperty( "LargestCWMatch" );
    If( !L_CW )
    SetGlobalProperty( "LargestCWMatch", TSize );
    L_CW := TSize;
    Endif
    
    If( TSize > L_CW )
    SetGlobalProperty( "LargestCWMatch", TSize );
    Endif

    Var iR := 0;
    Repeat
    iR := iR + 1;
    Var Res := CW_Red_Contestants.Append( Queued_Players[iR] );
    CW_Contestants.Append( Queued_Players[iR] );
    Var Red_Player := SystemFindCW( Queued_Players[iR] );
    If( Red_Player )
    SetObjProperty( Red_Player, CWID_INGAME, CWID_TEAMID_RED );
    Endif
    Queued_Players.Erase( iR );
    SleepMS( 10 );
    Until( iR >= TSize );
    
    Var iB := 0;
    Repeat
    iB := iB + 1;
    CW_Blue_Contestants.Append( Queued_Players[iB] );
    CW_Contestants.Append( Queued_Players[iB] );
    Var Blue_Player := SystemFindCW( Queued_Players[iB] );
    If( Blue_Player )
    SetObjProperty( Blue_Player, CWID_INGAME, CWID_TEAMID_BLUE );
    Endif
    SleepMS( 10 );
    Until( iB >= TSize );
    
    SetGlobalProperty( "CW_Red_Contestants", CW_Red_Contestants );
    SetGlobalProperty( "CW_Blue_Contestants", CW_Blue_Contestants );
    SetGlobalProperty( "CW_Contestants", CW_Contestants );
    SetGlobalProperty( CWID_INPROGRESS, 1 );
    
    ReadyCWContestants( CW_Contestants );
    
    Return 1;

Endfunction

Function ResurrectCWPlayer( Who )

    If( Who.dead )
    Resurrect( who );
    FixColor( who );
    SetStamina( Who, Cint( 35 + Random( 25 ) ) );
    SetHp( Who, Cint( 35 + Random( 25 ) ) );
    SetMana( Who, Cint( 35 + Random( 25 ) ) );
      Var DeathRobe := GetEquipmentByLayer( Who, LAYER_ROBE );
      If( DeathRobe )
      DestroyItem( DeathRobe );
      Endif
    Else
    Return Error;
    Endif

Endfunction

Function SendContestantToCWBase( Who )

    Var CWCFG_Arenas := ReadConfigFile( CW_CFG_ARENAS2 );
    Var iArena := 1;
    Var CW_Team := GetObjProperty( Who, CWID_INGAME );
    If( CW_Team == error )
    Print( "SCTB>CW_TEAM: " + CW_Team.errortext );
    Return Error;
    Endif

    If( CW_Team == CWID_TEAMID_RED )
    
    Var Red_Player_StartLoc := CWCFG_Arenas[iArena].Red_Start_Loc;
    
    If( Red_Player_StartLoc == error )
    Print( "Red_Player_StartLoc: " + Red_Player_StartLoc.errortext );
    Return Error;
    Endif
    
    Red_Player_StartLoc := SplitWords( Red_Player_StartLoc );
    MoveCharacterToLocation( Who, CInt( Red_Player_StartLoc[1] ), CInt( Red_Player_StartLoc[2] ), CInt( Red_Player_StartLoc[3] ), MOVECHAR_FORCELOCATION );
    Return 1;

    Elseif( CW_Team == CWID_TEAMID_BLUE )

    Var Blue_Player_StartLoc := CWCFG_Arenas[iArena].Blue_Start_Loc;
    
    If( Blue_Player_StartLoc == error )
    Print( "Blue_Player_StartLoc: " + Blue_Player_StartLoc.errortext );
    Return Error;
    Endif
    
    Blue_Player_StartLoc := SplitWords( Blue_Player_StartLoc );
    MoveCharacterToLocation( Who, CInt( Blue_Player_StartLoc[1] ), CInt( Blue_Player_StartLoc[2] ), CInt( Blue_Player_StartLoc[3] ), MOVECHAR_FORCELOCATION );
    Return 1;
    Endif

Endfunction

Function RemoveCWContestant( Who )

    Var CWCheck := GetObjProperty( Who, CWID_INGAME );
    Var CWCFG_Arenas := ReadConfigFile( CW_CFG_ARENAS );
    Var CW_Red_Contestants := GetGlobalProperty( "CW_Red_Contestants" );
    Var CW_Blue_Contestants := GetGlobalProperty( "CW_Blue_Contestants" );
    Var CW_Contestants := GetGlobalProperty( "CW_Contestants" );

    If( CWCheck )

    Var Hair := GetEquipmentByLayer( Who, LAYER_HAIR );
    Var Beard := GetEquipmentByLayer( Who, LAYER_BEARD );

    Foreach Worn in ListEquippedItems( Who )
    If( Worn && Worn != Hair && Worn != Beard && Worn != Who.backpack )
      If( GetObjProperty( Worn, "CW" ) || GetObjProperty( Worn, "CWRobe" ) )
      DestroyCWEquipment( Who, Worn );
      Endif
    Endif
    Endforeach
    
    Foreach Item in EnumerateItemsInContainer( Who.backpack );
    If( GetObjProperty( Item, "CW" ) || GetObjProperty( Item, "CWRobe" ) )
    DestroyCWEquipment( Who, Item );
    Endif
    Endforeach
    
    RemoveFromArray( CW_Red_Contestants, Who.serial );
    RemoveFromArray( CW_Blue_Contestants, Who.serial );
    RemoveFromArray( CW_Contestants, Who.serial );
    
    SetGlobalProperty( "CW_Red_Contestants", CW_Red_Contestants );
    SetGlobalProperty( "CW_Blue_Contestants", CW_Blue_Contestants );
    SetGlobalProperty( "CW_Contestants", CW_Contestants );
    
    Var xArena := 1;
    Var Main_StartLoc := CWCFG_Arenas[xArena].ArenaHome;
    Main_StartLoc := SplitWords( Main_StartLoc );
    MoveCharacterToLocation( Who, CInt( Main_StartLoc[1] ), CInt( Main_StartLoc[2] ), CInt( Main_StartLoc[3] ), MOVECHAR_FORCELOCATION );
    EraseObjProperty( Who, CASTINGOPT_PROP_NAME );
    EraseObjProperty( Who, CWID_INGAME );

    Endif

Endfunction

Function CheckCWContestant( Who )

    Var Hair := GetEquipmentByLayer( Who, LAYER_HAIR );
    Var Beard := GetEquipmentByLayer( Who, LAYER_BEARD );
    Var Item_Count := 0;
    Var PassGo := { LAYER_HAIR, LAYER_BEARD };
    Var xBankbox := FindBankBox( Who );

    Var Mount := GetEquipmentByLayer( Who, LAYER_MOUNT );
    
    If( Mount )
    Dismount( Who );
    Endif

    Foreach Worn in ListEquippedItems( Who )
    If( Worn && Worn != Hair && Worn != Beard && Worn != Who.backpack )
    MoveItemToContainer( Worn, xBankbox );
    SleepMS( 5 );
    Endif
    Endforeach

    Foreach Item in EnumerateItemsInContainer( Who.backpack );
    MoveItemToContainer( Item, xBankbox );
    SleepMS( 10 );
    Endforeach
    
    SendSysMessage( Who, "Everything you had on you was moved into your backbox.", FONT_NORMAL, 2611 );
    
    Return 1;

Endfunction

Function RewardCWTeam( CWTeam )
    
    Set_Critical( 1 );
    Var Cntstnt;

    Foreach Contestant in CWTeam
    Cntstnt := SystemFindCW( Contestant );
    Var xBankbox := FindBankBox( Cntstnt );
    CreateItemInContainer( xBankbox, 0xA396, CWID_COINREWARDAMT );
    Var CW_Wins := GetObjProperty( Cntstnt, "LT_CWWins" );
    If( !CW_Wins )
    CW_Wins := 0;
    Endif
    SetObjProperty( Cntstnt, "LT_CWWins", CW_Wins + 1 );
    SendSysMessage( Cntstnt, "Congrats, You've earned some unreal coins!", FONT_NORMAL, 2600 );
    SleepMS( 10 );
    Endforeach

    Set_Critical( 0 );

Endfunction

Function AwardCWPoint( Who, Points )

    Var Captures := GetObjProperty( Who, "LT_Captures" );
    If( !Captures )
    Captures := 0;
    Endif
    
    SetObjProperty( Who, "LT_Captures", Captures + Points );
    
    Return 1;

Endfunction

Function DestroyCWEquipment( Who, Item )

    Var Cursed := GetObjProperty( Item, "Cursed" );

    If( Cursed )
    SetObjProperty( Item, "Cursed", 3 );
    Endif

    If( Who.backpack )
    MoveItemToContainer( Item, Who.backpack );
    Endif

    DestroyItem( item );

Endfunction
/*
Function RemoveFromArray( byref x_array, match )

    Foreach index in ( x_array )
    If ( index == match )
    x_array.Erase( _index_iter );
    Return 1;
    Endif
    SleepMS( 2 );
    Endforeach
    Return Error;

Endfunction
*/
Function DressCWMage( player, TeamColor )
	var ccode := TeamColor;
    	var hat := CreateItemInBackpack( player, 0x1718, 1);
	hat.color := ccode;
        hat.newbie := 1;
	SetObjProperty(hat,"CW",1);
    	var robe := CreateItemInBackpack( player, 0x1F03, 1);
	robe.color := ccode;
	EquipItem( player, robe );
        robe.newbie := 1;
	SetObjProperty(robe,"CWRobe",1);
    	var bandies := CreateItemInBackpack( player, 0xe21, 500);
	bandies.color := ccode;
        bandies.newbie := 1;
	SetObjProperty(bandies,"CW",1);
	var book := CreateItemInBackpack(player,0x0efa, 1);
	book.color := ccode;
	SetObjProperty(book,"CW",1);
	var i;
	for (i := 0x1f2d; i <= 0x1f64; i := i + 1)
		if (spellcwok ( i )  )
			CreateItemInContainer(book,  i, 1);
		endif
	endfor
    	var staff := CreateItemInBackpack( player, 0xe8a, 1);
	staff.color := ccode;
        staff.newbie := 1;
	SetObjProperty(staff,"CW",1);
	staff.dmg_mod := 20;
	staff.name := "a Quarterstaff of Power[CW]";
	var bow := CreateItemInBackpack( player, 0x7600, 1);
	bow.color := ccode;
        bow.newbie := 1;
	SetObjProperty(bow,"CW",1);
	bow.dmg_mod := 15;
	bow.name := "Bow of Power[CW]";
	var arrow := CreateItemInBackpack( player, 0xf3f, 1000);
	arrow.color := ccode;
        arrow.newbie := 1;
	SetObjProperty(arrow,"CW",1);
	var boir := CreateItemInBackpack( player, 0xba30, 1 );
	boir.color := ccode;
        boir.newbie := 1;
	SetObjProperty( boir,"CW",1);
Endfunction

Function DressCWBard( player, TeamColor )

    Var ccode := TeamColor;
    Var hat := CreateItemInBackpack( player, 0x1718, 1 );
    hat.color := ccode;
    hat.newbie := 1;
    SetObjProperty( hat, "CW", 1 );
    Var robe := CreateItemInBackpack( player, 0x1F03, 1 );
    robe.color := ccode;
    EquipItem( player, robe );
    robe.newbie := 1;
    SetObjProperty( robe, "CWRobe", 1 );
    Var bandies := CreateItemInBackpack( player, 0xe21, 100 );
    bandies.color := ccode;
    bandies.newbie := 1;
    SetObjProperty( bandies, "CW", 1 );
    // Book of Songs
    Var bard := CreateItemInBackpack( player, 0x6177, 1 );
    SetObjProperty( bard , "Lesser" , 255 );
    SetObjProperty( bard , "Greater" , 255 );
    bard.color := ccode;
    SetObjProperty( bard, "CW", 1 );
    Var staff := CreateItemInBackpack( player, 0xe8a, 1 );
    staff.color := ccode;
    staff.newbie := 1;
    SetObjProperty( staff, "CW", 1 );
    staff.dmg_mod := 20;
    staff.name := "a Quarterstaff of Power[CW]";
    Var bow := CreateItemInBackpack( player, 0x7600, 1 );
    bow.color := ccode;
    bow.newbie := 1;
    SetObjProperty( bow, "CW", 1 );
    bow.dmg_mod := 15;
    bow.name := "Bow of Power[CW]";
    Var arrow := CreateItemInBackpack( player, 0xf3f, 1000 );
    arrow.color := ccode;
    arrow.newbie := 1;
    SetObjProperty( arrow, "CW", 1 );
    Var boir := CreateItemInBackpack( player, 0xba30, 1 );
    boir.color := ccode;
    boir.newbie := 1;
    SetObjProperty( boir, "CW",1 );
    Var harp := CreateItemInBackpack( player, 0xEB2, 1 );
    harp.color := ccode;
    harp.newbie := 1;
    SetObjProperty( harp, "CW",1 );
    
Endfunction

Function DressCWWarrior( player, TeamColor )
	var ccode := TeamColor;
    	var robe := CreateItemInBackpack( player, 0x1F03, 1);
	robe.color := ccode;
	EquipItem( player, robe );
        robe.newbie := 1;
	SetObjProperty( robe, "CWRobe", 1 );
    	var arms := CreateItemInBackpack( player, 0x1410, 1);
	arms.color := ccode;
        arms.newbie := 1;
	SetObjProperty(arms,"CW",1);
    	var legs := CreateItemInBackpack( player, 0x1411, 1);
	legs.color := ccode;
        legs.newbie := 1;
	SetObjProperty(legs,"CW",1);
    	var helm := CreateItemInBackpack( player, 0x1412, 1);
	helm.color := ccode;
        helm.newbie := 1;
	SetObjProperty(helm,"CW",1);
    	var gorg := CreateItemInBackpack( player, 0x1413, 1);
	gorg.color := ccode;
        gorg.newbie := 1;
	SetObjProperty(gorg,"CW",1);
    	var glove := CreateItemInBackpack( player, 0x1414, 1);
	glove.color := ccode;
        glove.newbie := 1;
	SetObjProperty(glove,"CW",1);
	var bp := CreateItemInBackpack( player, 0x1415, 1);
	bp.color := ccode;
        bp.newbie := 1;
	SetObjProperty(bp,"CW",1);
    	var heater := CreateItemInBackpack( player, 0x1b76, 1);
	heater.color := ccode;
        heater.newbie := 1;
	SetObjProperty(heater,"CW",1);
    	var bandies := CreateItemInBackpack( player, 0xe21, 500 );
	bandies.color := ccode;
        bandies.newbie := 1;
	SetObjProperty(bandies,"CW",1);
	var kryss := CreateItemInBackpack( player, 0x1401, 1);
	kryss.color := ccode;
	kryss.dmg_mod := 20;
        kryss.newbie := 1;
	SetObjProperty(kryss,"CW",1);
	kryss.name := "Kryss of Power[CW]";
    	var mace := CreateItemInBackpack( player, 0xf5c, 1);
	mace.color := ccode;
	SetObjProperty(mace,"CW",1);
	mace.dmg_mod := 20;
        mace.newbie := 1;
	mace.name := "Mace of Power[CW]";
    	var sword := CreateItemInBackpack( player, 0xf5e, 1);
	sword.color := ccode;
        sword.newbie := 1;
	SetObjProperty(sword,"CW",1);
	sword.dmg_mod := 20;
	sword.name := "Broad Sword of Power[CW]";
    	var spear := CreateItemInBackpack( player, 0xf63, 1);
	spear.color := ccode;
        spear.newbie := 1;
	SetObjProperty(spear,"CW",1);
	spear.dmg_mod := 20;
	spear.name := "Spear of Power[CW]";
	var book := CreateItemInBackpack(player,0x0efa, 1);
	book.color := ccode;
	SetObjProperty(book,"CW",1);
	var i;
	for (i := 0x1f2d; i <= 0x1f64; i := i + 1)
		if (spellCWok ( i )  )
			CreateItemInContainer(book,  i, 1);
		endif
	endfor
	var bow := CreateItemInBackpack( player, 0x7600, 1);
	bow.color := ccode;
        bow.newbie := 1;
	SetObjProperty(bow,"CW",1);
	bow.dmg_mod := 15;
	bow.name := "Bow of Power[CW]";
	var arrow := CreateItemInBackpack( player, 0xf3f, 1000);
	arrow.color := ccode;
        arrow.newbie := 1;
	SetObjProperty(arrow,"CW",1);
	var boir := CreateItemInBackpack( player, 0xba30, 1 );
	boir.color := ccode;
        boir.newbie := 1;
	SetObjProperty( boir,"CW",1);
Endfunction

Function DressCWRanger( player, TeamColor )
	var ccode := TeamColor;
    	var robe := CreateItemInBackpack( player, 0x1F03, 1);
	robe.color := ccode;
	EquipItem( player, robe );
        robe.newbie := 1;
	SetObjProperty( robe, "CWRobe", 1 );
    	var arms := CreateItemInBackpack( player, 0x13ee, 1);
	arms.color := ccode;
        arms.newbie := 1;
	SetObjProperty(arms,"CW",1);
    	var legs := CreateItemInBackpack( player, 0x13be, 1);
	legs.color := ccode;
        legs.newbie := 1;
	SetObjProperty(legs,"CW",1);
    	var helm := CreateItemInBackpack( player, 0x13c0, 1);
	helm.color := ccode;
        helm.newbie := 1;
	SetObjProperty(helm,"CW",1);
    	var glove := CreateItemInBackpack( player, 0x13f2, 1);
	glove.color := ccode;
        glove.newbie := 1;
	SetObjProperty(glove,"CW",1);
	var bp := CreateItemInBackpack( player, 0x13bf, 1);
	bp.color := ccode;
        bp.newbie := 1;
	SetObjProperty(bp,"CW",1);
	var boots := CreateItemInBackpack( player, 0x170b, 1);
	boots.color := ccode;
        boots.newbie := 1;
	SetObjProperty(boots,"CW",1);
    	var bandies := CreateItemInBackpack( player, 0xe21, 100);
	bandies.color := ccode;
        bandies.newbie := 1;
	SetObjProperty(bandies,"CW",1);
	var bow := CreateItemInBackpack( player, 0x7600, 1);
	bow.color := ccode;
        bow.newbie := 1;
	SetObjProperty(bow,"CW",1);
	bow.dmg_mod := 25;
	bow.name := "Bow of Vanq[CW]";
	var arrow := CreateItemInBackpack( player, 0xf3f, 2000 );
	arrow.color := ccode;
        arrow.newbie := 1;
	SetObjProperty(arrow,"CW",1);
	var pot;
	var potloop := 0;
	while (potloop < 5)
		potloop := potloop + 1;
		pot := CreateItemInBackpack (player ,0xdc02, 1);
          pot.newbie := 1;
		SetObjProperty(pot,"CW",1);
	endwhile
 
	var book := CreateItemInBackpack(player,0x0efa, 1);
	book.color := ccode;
	SetObjProperty(book,"CW",1);
	var i;
	for (i := 0x1f2d; i <= 0x1f64; i := i + 1)
		if (spellCWok ( i )  )
			CreateItemInContainer(book,  i, 1);
		endif
	endfor
	var boir := CreateItemInBackpack( player, 0xba30, 1 );
	boir.color := ccode;
        boir.newbie := 1;
	SetObjProperty( boir,"CW",1);
Endfunction

Function DressCWPP( player, TeamColor )
	var ccode := TeamColor;
    	var robe := CreateItemInBackpack( player, 0x1F03, 1);
	robe.color := ccode;
	EquipItem( player, robe );
        robe.newbie := 1;
	SetObjProperty( robe, "CWRobe", 1 );
    	var arms := CreateItemInBackpack( player, 0x1410, 1);
	arms.color := ccode;
        arms.newbie := 1;
	SetObjProperty(arms,"CW",1);
    	var legs := CreateItemInBackpack( player, 0x1411, 1);
	legs.color := ccode;
        legs.newbie := 1;
	SetObjProperty(legs,"CW",1);
    	var helm := CreateItemInBackpack( player, 0x1412, 1);
	helm.color := ccode;
        helm.newbie := 1;
	SetObjProperty(helm,"CW",1);
    	var gorg := CreateItemInBackpack( player, 0x1413, 1);
	gorg.color := ccode;
        gorg.newbie := 1;
	SetObjProperty(gorg,"CW",1);
    	var glove := CreateItemInBackpack( player, 0x1414, 1);
	glove.color := ccode;
        glove.newbie := 1;
	SetObjProperty(glove,"CW",1);
	var bp := CreateItemInBackpack( player, 0x1415, 1);
	bp.color := ccode;
        bp.newbie := 1;
	SetObjProperty(bp,"CW",1);
    	var heater := CreateItemInBackpack( player, 0x1b76, 1);
	heater.color := ccode;
        heater.newbie := 1;
	SetObjProperty(heater,"CW",1);
    	var bandies := CreateItemInBackpack( player, 0xe21, 500);
	SetObjProperty(bandies,"CW",1);
        bandies.newbie := 1;
	bandies.color := ccode;
	var book := CreateItemInBackpack(player,0x0efa, 1);
	book.color := ccode;
	SetObjProperty(book,"CW",1);
	var i;
	for (i := 0x1f2d; i <= 0x1f64; i := i + 1)
		if (spellCWok ( i )  )
			CreateItemInContainer(book,  i, 1);
		endif
	endfor
    	var kryss := CreateItemInBackpack( player, 0x1401, 1);
	kryss.color := ccode;
	kryss.dmg_mod := 20;
        kryss.newbie := 1;
	kryss.name := "Kryss of Power[CW]";
	SetObjProperty(kryss,"CW",1);
    	var mace := CreateItemInBackpack( player, 0xf5c, 1);
	mace.color := ccode;
	mace.dmg_mod := 20;
        mace.newbie := 1;
	mace.name := "Mace of Power[CW]";
	SetObjProperty(mace,"CW",1);
    	var sword := CreateItemInBackpack( player, 0xf5e, 1);
	sword.color := ccode;
	sword.dmg_mod := 20;
        sword.newbie := 1;
	sword.name := "Broad Sword of Power[CW]";
	SetObjProperty(sword,"CW",1);
    	var spear := CreateItemInBackpack( player, 0xf63, 1);
	spear.color := ccode;
        spear.newbie := 1;
	SetObjProperty(spear,"CW",1);
	spear.dmg_mod := 20;
	spear.name := "Spear of Power[CW]";
	var bow := CreateItemInBackpack( player, 0x7600, 1);
	bow.color := ccode;
        bow.newbie := 1;
	SetObjProperty(bow,"CW",1);
	bow.dmg_mod := 15;
	bow.name := "Bow of Power[CW]";
	var arrow := CreateItemInBackpack( player, 0xf3f, 1000 );
	arrow.color := ccode;
        arrow.newbie := 1;
	SetObjProperty(arrow,"CW",1);
 
	var boir := CreateItemInBackpack( player, 0xba30, 1 );
	boir.color := ccode;
        boir.newbie := 1;
	SetObjProperty( boir,"CW",1);
 
Endfunction

Function SpellCWOk( what )

    Case( what )
    0x1F41:
    0x1F42:
    0x1F44:
    0x1F4C:
    0x1F4D:
    0x1F4F:
    0x1F54:
    0x1F60:
    0x1F66:
    0x1F67:
    0x1F68:
    0x1F69:
    0x1F6A:
    0x1F6B:
    0x1F6C: Return 0;
    default: Return 1;
    Endcase

Endfunction
