use uo;

include "include/client";
include "include/dotempmods";
include "include/classes";
include "include/damages";
include "include/bitwise";
include "include/random";

var itemcfgfile := ReadConfigFile(":combat:itemdesc");


function RecalcDmg( attacker, defender, weapon, armor, byref basedamage, piercing := 0 )

    return RecalcPhysicalDmg( attacker, defender, weapon, armor, basedamage, piercing );

endfunction

function SetAstralIncapacity( who )

	start_script( ":astralfights:astralincapacity", who );

endfunction

function RecalcPhysicalDmg( attacker, defender, weapon, armor, byref basedamage, piercing := 0 )

	var rawdamage;
	var absorbed;
	var ar := CInt(armor.ar) + CInt(defender.ar_mod);
	var anatomy := GetAttribute( attacker, ATTRIBUTEID_ANATOMY );

	if( defender.isA(POLCLASS_NPC) )
		if( !attacker.npctemplate )
			basedamage := basedamage * 1.5;
		endif
	endif

        basedamage := ModByDist( attacker, defender, weapon, basedamage );
	basedamage := basedamage * ( 1 + anatomy * 0.002 );

	if( !piercing )
		rawdamage := GetShieldAbsorption( attacker, defender, weapon, basedamage );
		absorbed  := ar * (Random(51) + 50) * 0.01;
	else
		rawdamage := basedamage;
		absorbed  := 0;
	endif

	rawdamage := rawdamage - absorbed;
	rawdamage := rawdamage * 0.5;
	rawdamage := ModByProt( defender, rawdamage );

	if( rawdamage < 0 )
		rawdamage := 0;
	endif

	MaceHit( attacker, defender, weapon, armor, absorbed );
//	CheckForTheSkillGain( attacker , defender , weapon );

	return CInt( rawdamage );

endfunction


function RecalcAstralDmg( attacker, defender, weapon, armor, byref basedamage, piercing := 0 )

	var rawdamage;
	var absorbed := 0;

	var ar := CInt( GetObjProperty( armor, "Astral" ) ) * 25 * CInt( armor.ar );

	basedamage := basedamage
		* ((GetAttribute( attacker, ATTRIBUTEID_SPIRITSPEAK ) + 50.0 + (GetIntelligence(attacker) * 0.2)) * 0.01)
		/ ((GetAttribute( attacker, ATTRIBUTEID_TACTICS ) + 50.0 + (GetStrength(attacker) * 0.2)) * 0.01);

	AwardRawSkillPoints( attacker, SKILLID_EVALINT, 30 );
	AwardRawSkillPoints( attacker, SKILLID_SPIRITSPEAK, 30 );

	if( defender.isA(POLCLASS_NPC) )
		basedamage := basedamage * ClasseBonus( attacker, CLASSEID_MAGE );
	else
		var level := GetObjProperty( attacker, CLASSEID_MAGE );
		if( level )
			level := level - 2;
			if( level >= 1 )
				basedamage := basedamage * ClasseBonusByLevel( level );
			endif
		endif
	endif

	if( GetObjProperty( attacker, CLASSEID_WARRIOR ) )
		if( !GetObjProperty( attacker, CLASSEID_MAGE ) )
			basedamage := basedamage / ClasseBonus( attacker, CLASSEID_WARRIOR );
		endif
	endif

	if( GetObjProperty( defender, CLASSEID_WARRIOR ) )
		if( !GetObjProperty( defender, CLASSEID_MAGE ) )
			basedamage := basedamage * ClasseBonus( defender, CLASSEID_WARRIOR );
		endif
	endif

	basedamage := basedamage * ( 1 + GetAttribute( attacker, ATTRIBUTEID_EVALINT ) * 0.002 );
	rawdamage  := basedamage;

	if( !piercing )

		var meditation := GetAttribute( defender, ATTRIBUTEID_MEDITATION );
		meditation := meditation * ClasseBonus( defender, CLASSEID_MAGE );

		if( (Random(100) + 1) <= CInt( meditation * 0.5 ) )

			var resist := GetAttribute( defender, ATTRIBUTEID_MAGICRESISTANCE );
			var defint := GetIntelligence(defender);

			AwardRawSkillPoints( defender, SKILLID_MEDITATION, 30 );
			AwardRawSkillPoints( defender, SKILLID_MAGICRESISTANCE, 30 );
			PerformAction(  defender, ANIM_CAST_AREA );
			SendSysMessage( defender, "You successfully block a part of the astral attack." );

			if( GetObjProperty( defender, CLASSEID_MAGE ) )
				rawdamage := rawdamage - (resist + defint) * 0.25 * ClasseBonus( defender, CLASSEID_MAGE );
			elseif( GetObjProperty( defender, CLASSEID_WARRIOR ) )
				rawdamage := rawdamage - (resist + defint) * 0.25 / ClasseBonus( defender, CLASSEID_WARRIOR );
			endif

			if( rawdamage < 0 )
				rawdamage := 0;
			endif

		endif

		if( ar )
			absorbed := CInt( ar * (Random(51) + 50) / 100 );
		endif

	endif

	rawdamage := rawdamage - absorbed;
	rawdamage := rawdamage * 0.5;
	if( rawdamage < 0 )
		rawdamage := 0;
	endif

//	CheckForTheSkillGain( attacker , defender , weapon );

	return CInt(rawdamage);

endfunction


function ModByDist( attacker, defender, weapon, basedamage )

	if( weapon.attribute == "Archery" )
		basedamage := basedamage * ((GetDexterity(attacker) + 60.0) * 0.01 )
			/ ((GetAttribute( attacker, ATTRIBUTEID_TACTICS ) + 50.0 + (GetStrength(attacker) / 5.0)) * 0.01);

		var dist := Distance( attacker, defender );
		if( (dist <= 1) || (dist > 10) )
			basedamage := basedamage * 0.25;
		endif

	if( GetObjProperty( attacker, CLASSEID_RANGER ) )
 
            if( GetAttribute( attacker, "Tactics" ) <= 1 )
            basedamage := basedamage * ( ( GetAttribute( attacker, "Archery" ) + 50 ) * 0.005 );
            endif
            
			var level := GetObjProperty( attacker, CLASSEID_RANGER );
			if( level )
				level := level - 2;
				if( level >= 1 )
					basedamage := basedamage * ClasseBonusByLevel( level );
				endif
			endif

	endif

	else

		if( defender.isA(POLCLASS_NPC) )
			basedamage := basedamage * ClasseBonus( attacker, CLASSEID_WARRIOR );
		else
			var level := GetObjProperty( attacker, CLASSEID_WARRIOR );
			if( level )
				level := level - 2;
				if( level >= 1 )
					basedamage := basedamage * ClasseBonusByLevel( level );
				endif
			endif
		endif

	endif

	if( GetObjProperty( attacker, CLASSEID_MAGE ) )
		if( !GetObjProperty( attacker, CLASSEID_WARRIOR ) )
			basedamage := basedamage / ClasseBonus( attacker, CLASSEID_MAGE );
		endif
	endif

	if( GetObjProperty( defender, CLASSEID_MAGE ) )
		if( !GetObjProperty( defender, CLASSEID_WARRIOR ) )
			basedamage := basedamage * ClasseBonus( defender, CLASSEID_MAGE );
		endif
	endif

	return basedamage;

endfunction


function MaceHit( attacker, defender, weapon, armor, absorbed )

	var archance := 2;
	var weapchance := 2;

	if( weapon.attribute == "Mace" )

		var loss := Random(2) + 1;
		if(loss > GetStamina(defender) )
			SetStamina(defender, 0);
		else
			SetStamina(defender, GetStamina(defender) - loss);
		endif

		if( armor.hp )
			if( Random(100) < 5 )
				armor.hp := armor.hp - 1;
			endif
		endif

	elseif( Random(100) < archance )

		if( armor.hp )
			armor.hp := armor.hp - 1;
		endif

	endif

	if( Random(100) < weapchance )
		if( weapon.hp )
			weapon.hp := weapon.hp - 1;
		endif
	endif

	if( armor.quality )
		if( (armor.hp > armor.maxhp) || (armor.hp < 1) )
			DestroyEquipment( defender, armor );
		endif
	endif

	if( weapon.quality )
		if( (weapon.hp > weapon.maxhp) || (weapon.hp < 1) )
			DestroyEquipment( attacker, weapon );
		endif
	endif

endfunction


function ModByProt( defender, basedamage )

	var physicalprot := GetObjProperty( defender, "PhysicalProtection" );
	if( !physicalprot )
		return basedamage;
	else
		basedamage := basedamage * ( 1 - physicalprot * 0.05 );
		return basedamage;
	endif

endfunction


function GetShieldAbsorption( attacker, defender, weapon, basedamage )

	var shield := GetEquipmentByLayer( defender, LAYER_HAND2 );
	var shieldar := CInt(shield.ar);
	if( shieldar )
		var parry_skill := GetAttribute( defender, ATTRIBUTEID_PARRY );
		parry_skill := CInt( parry_skill * ClasseBonus( attacker, CLASSEID_WARRIOR ) );

		if( (Random(100) + 1) <= CInt(parry_skill / 2) )
			var absorbed;

			PerformAction( defender, ANIM_ATTACK_XBOW );
			SendSysMessage( defender, "You successfully parry the attack." );

			if( weapon.attribute == ATTRIBUTEID_ARCHERY )
				absorbed := shieldar;
			else
				absorbed := CInt(shieldar / 2);
			endif

			basedamage := basedamage - absorbed;
			if( basedamage < 0 )
				basedamage := 0;
			endif

			MaceHit( attacker, defender, weapon, shield, absorbed );
		endif
	endif

	return basedamage;

endfunction


function DealDamage( attacker, defender, weapon, armor, basedamage, rawdamage )

	if( !GetObjProperty( weapon, "Astral" ) )

		var poisoned := GetObjProperty( weapon, "SkillPoisoned" );
		var reactive := GetObjProperty( defender, "ReactiveArmor" );
		var onhit    := GetObjProperty( armor, "OnHitScript" );
			//Check if the item was poisonned by someone
		if( poisoned )

			var plvl := CInt( poisoned / 3 );
			poisoned  := poisoned - 1;
			if( poisoned )
				SetObjProperty( weapon, "SkillPoisoned", poisoned );
			else
				EraseObjProperty( weapon, "SkillPoisoned" );
			endif

			SetPoison( defender, plvl, "You have been poisoned by " + attacker.name + "." );

		endif

		if( GetObjProperty( defender, CLASSEID_WARRIOR ) )
			if( attacker.isA(POLCLASS_NPC) )
				rawdamage := CInt( rawdamage / ClasseBonus( defender, CLASSEID_WARRIOR ) );
			else
				var level := GetObjProperty( defender, CLASSEID_WARRIOR );
				if( level )
					level := level - 1;
					if( level >= 1 )
						basedamage := CInt( basedamage / ClasseBonusByLevel( level ) );
						rawdamage := CInt( rawdamage / ClasseBonusByLevel( level ) );
					endif
				endif
			endif
		endif

		//Check if reactive armor spell is on
		if( reactive )
			start_script( ":combat:reactivearmoronhit", { attacker, defender, basedamage, rawdamage, reactive} );
		endif

		//Check if the armor have an on hit script and launch it
		if( onhit )
			start_script( onhit, {attacker, defender, weapon, armor, basedamage, rawdamage} );
		else
			ApplyTheDamage( defender, attacker, rawdamage );
		endif

	else

		ApplyTheAstralDamage( defender, attacker, rawdamage );

	endif

endfunction


function DestroyEquipment( who, item )

	var cursed := GetObjProperty( item, "Cursed" );

	if( cursed )
		SetObjProperty( item, "Cursed", 3 );
	endif

	if( who.backpack )
		MoveItemToContainer( item, who.backpack );
	endif

	if( who.isA(POLCLASS_NPC) )
		if( item.attribute )
			PrintTextAbove( who, who.name + " seem exhausted and can barely hit anyones anymore!" );
		else
			PrintTextAbove( who, who.name + "'s defences has been penetrated making it easy to hit vital parts!" );
		endif
	endif

	DestroyItem( item );

endfunction
